<?php

/**
 * Piwik - Open source web analytics
 *
 * @link http://www.suenkel.de/
 * @version $Id$
 * @author Christian Suenkel
 *
 * The GeoIPMap-plugin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * The GeoIPMap-plugin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GeoIPMap.  If not, see <http://www.gnu.org/licenses/>.
 * The GeoIPMap-Plugin is free software: you can redistribute it and/or modify
 *
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GeoIPMap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GeoIPMap.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @category Piwik_Plugins
 * @package Piwik_GeoIPMap
 */

/**
 *  GeoIPMap - Plugin
 *
 *  provides a visual interface (based on google-maps v3) to the GeoIP-plugin
 *  @see http://dev.piwik.org/trac/ticket/45
 */
class Piwik_GeoIPMap extends Piwik_Plugin {
    /**
     * @var int - maximum of rows to be stored in the archive
     */
    const MAX_ROWS_TOARCHIVE = 100;

    /**
     * provide information about the plugin
     * @param void
     * @return array
     */
    public function getInformation() {
    	
        return array(
            'name' => 'GeoIP Google Map',
            'description' => 'Diplay the Location of your visitors in Googlemaps/OpenStreetMaps. It requires the plugin GeoIP',
            'author' => 'Christian Suenkel',
            'author_homepage' => 'http://www.suenkel.de/',
            'homepage' => 'http://piwik.org',
            'version' => '1.7.1',
            'TrackerPlugin' => false,
            'GeoIPPlugin' => true, // this plugin must be loaded during the stats logging
        );
    }

    /**
     * provide a List of Hooks to be registered in Piwik-eventhandling
     * @param void
     * @return array
     */
    public function getListHooksRegistered() {
        return array(
            'Menu.add' => 'addMenus',
            'ArchiveProcessing_Day.compute' => 'archiveDay',
            'ArchiveProcessing_Period.compute' => 'archivePeriod',
            'API.getReportMetadata' => 'getReportMetadata',
        // 'API.getSegmentsMetadata' => 'getSegmentsMetadata',
        );
    }

    /**
     * Hook to add the Menues beneath the visitors-menue
     * @param void
     * @return void
     */
    public function addMenus() {
        Piwik_AddMenu('General_Visitors',
                'Map', array('module' => 'GeoIPMap', 'action' => 'index'),
                true, 15);
        // Piwik_AddMenu('General_Visitors','MapLive', array('module' => 'GeoIPMap', 'action' => 'live'), true, 16);
    }

    /**
     * Apply information-array of the availability of segmentation
     * @param Piwik_Event_Notification $notification - the object of interes
     * @return void
     */
    public function getSegmentsMetadata($notification) {
        $segments = & $notification->getNotificationObject();
        $segments[] = array(
            'type' => 'dimension',
            'category' => 'Visit',
            'name' => Piwik_Translate('UserCountry_Country'),
            'segment' => 'country',
            'sqlSegment' => 'location_geoip_country',
            'acceptedValues' => 'de, us, fr, in, es, etc.'
        );
    }

    /**
     * Registers reports metadata information
     * @param Piwik_Event_Notification $notification - the object of interes
     * @return void
     */
    public function getReportMetadata($notification) {
        $reports = &$notification->getNotificationObject();
        $reports[] = array(
            'category' => Piwik_Translate('General_Visitors'),
            'name' => Piwik_Translate('UserCountry_Country'),
            'module' => 'GeoIPMap',
            'action' => 'index',
            'dimension' => Piwik_Translate('UserCountry_Country')
        );
    }

    /**
     * Compute the aggregation of a period (week, mont, year) and archive the data
     *
     * @param Piwik_ArchiveProcessing $notification - the object of interes
     * @return void
     */
    public function archivePeriod($notification) {
        $archiveProcessing = $notification->getNotificationObject();
        if (!$archiveProcessing->shouldProcessReportsForPlugin($this->getPluginName()))
            return;

        $maximumRowsInDataTable = Zend_Registry::get('config')->General->datatable_archiving_maximum_rows_standard;
        $maximumRowsInDataTable = min(self::MAX_ROWS_TOARCHIVE, $maximumRowsInDataTable);

        $archiveProcessing->archiveDataTable(array('GeoIPMap_map'),
                array(Piwik_Archive::INDEX_SUM_DAILY_NB_UNIQ_VISITORS => Piwik_Archive::INDEX_NB_UNIQ_VISITORS),
                $maximumRowsInDataTable, 0, Piwik_Archive::INDEX_NB_UNIQ_VISITORS);
    }

         
    /**
     * Compute the statistics of a day
     * 
     * @param Piwik_ArchiveProcessing $notification - the object of interes
     * @return void
     */
    public function archiveDay($notification) {

    	$archiveProcessing = $notification->getNotificationObject();
    	
    	if (!$archiveProcessing->shouldProcessReportsForPlugin($this->getPluginName())) return;

        $maximumRowsInDataTable = Zend_Registry::get('config')->General->datatable_archiving_maximum_rows_standard;
        $maximumRowsInDataTable = min(self::MAX_ROWS_TOARCHIVE, $maximumRowsInDataTable);

        
        
        /*
         * Handling a custom segment when processing page reports
         */
        $segment = $archiveProcessing->getSegment();

        /*
         * generate daily stats based on GeoIP-Data
         * (nb_visits and nb_uniq-visitors)
         */
        $select = "location_geoip_continent,
                            location_geoip_country,
		            location_geoip_city,
                            location_geoip_latitude,
         		    location_geoip_longitude,
			    count(distinct idvisitor) as `" . Piwik_Archive::INDEX_NB_UNIQ_VISITORS . "`,
			    count(*) as `" . Piwik_Archive::INDEX_NB_VISITS . "`";
        $from =  'log_visit'; //  Piwik_Common::prefixTable('log_visit');
        
        $where = "visit_last_action_time >= ?
			    AND visit_last_action_time <= ?
			    AND idsite = ?
                            and location_geoip_continent !=''
                            and location_geoip_country !=''";
        $group ="location_geoip_continent, location_geoip_country, location_geoip_city, location_geoip_latitude,location_geoip_longitude";
        $order = "`" . Piwik_Archive::INDEX_NB_UNIQ_VISITORS . "` DESC LIMIT $maximumRowsInDataTable";
        $sqlBind = array(
                    $archiveProcessing->getStartDatetimeUTC(),
                    $archiveProcessing->getEndDatetimeUTC(),
                    $archiveProcessing->idsite);
                    
        $segmentSql = $segment->
                        getSelectQuery($select, $from, $where, $sqlBind, $order, $group);
                   //   file_put_contents('/tmp/sql', $segmentSql['sql']."\n", FILE_APPEND);   
        $query = $archiveProcessing->db->query($segmentSql['sql'], $segmentSql['bind']);

        /**
         * convert the result of the database query to piwik-table
         */
        $table = new Piwik_DataTable();
        foreach ($query as $row) {
            $r = new Piwik_DataTable_Row(array(
                        Piwik_DataTable_Row::COLUMNS => array(
                            'label' => $row['location_geoip_latitude'] . '|' . $row['location_geoip_longitude'],
                            Piwik_Archive::INDEX_NB_UNIQ_VISITORS => $row[Piwik_Archive::INDEX_NB_UNIQ_VISITORS],
                            Piwik_Archive::INDEX_NB_VISITS => $row[Piwik_Archive::INDEX_NB_VISITS],
                        ),
                        Piwik_DataTable_Row::METADATA => array(
                            'location_geoip_continent' => $row['location_geoip_continent'],
                            'location_geoip_country' => $row['location_geoip_country'],
                            'location_geoip_city' => $row['location_geoip_city'])
                    ));

            $table->addRow($r);
        }
        $archiveProcessing->insertBlobRecord('GeoIPMap_map', $table->getSerialized());
        destroy($table);
    }

}

